
/* this .csd loads an audio file which contains pulses but no
   clear rhythmic beats, and replaces samples within the file
   with a percussive sound using Vaget/Vaset.  
   It uses a UDO and a logarithmic fade out of 1000 samples
   prior to the the sample replacement.
*/


<CsoundSynthesizer>
<CsOptions>

-s -d -+rtaudio=PortAudio -odac4 -W -b1024 -B16384

</CsOptions>
<CsInstruments>

sr=48000
ksmps=576000
nchnls=1

;GLOBAL VARIABLES
gitmp1    ftgen 100, 0, -576000, -10, 1
gitmp3   ftgen 200, 0, -576000, -10, 1
gitmp4   ftgen 300, 0, -576000, -10, 1

gabells init 0
gabells2 init 0
gaclank init 0

gilen1 filelen "Bells.wav"
gilen2 filelen "Clank.wav"
gisamples1 init 0
gisamples2 init 0

;=======================
;=======================
;UDO
opcode SampleReplace, a, iiiiiii

;init local variables
a1 init 0
a2 init 0
a3 init 0
kval init 0
kndx = 0
clear gabells


istart, ifade, icut, ismplstart, ismplend, icont, iend, xin

/* ;ck values, etc.
printf "start %d\n", 1, istart
printf "fade %d\n", 1, ifade
printf "cut %d\n", 1, icut
printf "smplRplstart > %d\n", 1, ismplstart
printf "smplRplend < %d\n", 1, ismplend
printf "cont %d\n", 1, icont
*/

a1 diskin "Samplereplace.wav", 1, 0, 0, 4
a2 diskin "clankfile.wav", 1, 0, 0, 4

loopStart1:

	;initial file
	if (kndx > istart && kndx < ifade) then
	kval vaget kndx,a1
	
	;SLIGHT FADE OUT within intial file
	elseif (kndx > ifade && kndx < icut) then
	kval vaget kndx,a1
	;LOGARITHMIC w xtra scaling
	kval = kval * (log10(10 - (10*((kndx-ifade)/1110)))) 
	
	;INSERT THE FIRST SAMPLE REPLACEMENT
	elseif (kndx > ismplstart && kndx < ismplend) then
	kval vaget kndx - ismplstart, a2
	
	;CONTINUE AFTER SAMP REPLACEMENT WITH ORIGINAL FILE
	elseif (kndx > icont) then
	kval vaget kndx, a1
	endif
	
	vaset kval,kndx,a3
	
	if (kndx > iend) goto last;	
	loop_lt kndx, 1, ksmps, loopStart1

last:
	gabells = gabells + a3
	printf "UDO finished at  %d\n", 1, kndx
	clear a1, a2, a3
	xout a3
endop
;====================
;====================
;====================
;==========================
;WRITE DISKIN FILES TO EQUAL LENGTH TABLES
  instr 1
;GET/WRITE Bells.wav to table(12 sec., no real beats, but pulsing snds)
;Load into a table(100)
  
print gilen1
gisamples1 = gilen1 * sr
print gisamples1

andx line 0, gilen1, gisamples1
asig1 diskin "Bells.wav", 1, 0, 0, 4
tablew asig1, andx, 100 
    endin
;==============
    instr 2
;GET/WRITE CLANK wav to table

andx line 0, gilen1, gisamples1 ;using length from above

asig2 diskin "Clank.wav", 1, 0, 0, 4
tablew asig2, andx, 200
    endin
;======================
;======================
;SAVE THE TABLES TO .WAV ON DISK
	instr 3
andx line 0, gilen1, gisamples1 ;using length from above
a1 table andx, 100
gabells= gabells + a1
	endin

	instr 4
;fout "/opt/csound5109/JFiles/Samplereplace.wav", 2, gabells
fout "Samplereplace.wav", 2, gabells
	endin
;==================
	instr 5
clear gabells
andx line 0, gilen1, gisamples1 ;using length from above
a1 table andx, 200

gaclank= gaclank + a1
	endin

	instr 6
;fout "/opt/csound5109/JFiles/clankfile.wav", 2, gaclank
fout "clankfile.wav", 2, gaclank
	endin
;=================
;=================
;======================
;======================
;READ FILES FROM DISK AND DO VAGET/VASET

	instr 7
;SAMPLE REPLACER
;istart, ifade, icut, ismplstart, ismplend, icont, iend, xin

ibeat = p4 ;beat to replace

istart = 0
ifade = (ibeat - 1000)-1
icut  =  ibeat -1
ismplstart = ibeat -1 
ismplend  = (ibeat - 1) + (gilen2 * sr)
icont = (ibeat - 1) + (gilen2 * sr)
iend = gisamples1 

asig SampleReplace istart, ifade, icut, ismplstart, ismplend, icont, iend
	endin

/*
SR = 48000
start = 0
2 sec. = 96000
4 sec. = 192000
8 sec. = 384000
10 sec. = 480000
12 sec. = EOF 576000
*/
;=================
;=================
	
;WRITE THE MODIFIED FILE TO DISK
	instr 8
;fout "/opt/csound5109/JFiles/Samplereplace.wav", 2, gabells
fout "Samplereplace.wav", 2, gabells
	endin
;=================
;=================

</CsInstruments>
<CsScore>

f1 0 -576000 1 "Bells.wav" 0 4 0 
f2 0 -24000 1 "Clank.wav" 0 4 0 	

i1	0	12
s
i2	0	12
s
i3	0	12
s
i4	0	12
s
i5	0	12
s
i6	0	12
s
i7	0	12	96000
s
i8	0	12
s
i7	0	12	192000
s
i8	0	12
s
i7	0	12	384000
s
i8	0	12
s
i7	0	12	480000
s
i8	0	12

e


</CsScore>
</CsoundSynthesizer>

